<?php

/**
 * DRD Server Operation to find out all domains on thsi Drupal installation.
 *
 * @return type
 */
function drd_server_server_domains() {
  $sites = drd_server_read_sites();
  $output = array();
  $error = array();
  foreach ($sites as $url => $id) {
    if (isset($output[$id])) {
      $output[$id]['aliases'][] = $url;
    }
    else if (isset($error[$id])) {
      // Ignore this alias, it won't work
    }
    else {
      $code = drd_server_server_check_host($url, $id);
      if ($code === FALSE) {
        $error[$id] = TRUE;
      }
      else {
        $output[$id] = array(
          'url' => $url,
          'aliases' => array(),
          'drd installed' => ($code === TRUE),
        );
      }
    }
  }
  return drd_server_result('server.domains', $output);
}

/**
 * Function called by drd_server_server_domains() to check a specific URL
 * if it really exists and if the module drd_server is installed.
 *
 * @param type $url
 * @param type $dir
 * @return boolean
 */
function drd_server_server_check_host($url, $dir) {
  $id = rand();
  try {
    $check = drupal_http_request('http://' . $url . '/admin/drd_server/' . $id);
    $ok = (($check->code == 200) || ($check->code == 302 && $check->redirect_code == 200));
    if ($ok && $check->data == ($id / 2)) {
      return TRUE;
    }
    return $check->code;
  }
  catch (Exception $e) {
    return FALSE;
  }
}

function drd_server_server_list_updates() {
  if (!module_exists('update')) {
    module_enable(array('update'));
  }
  $org_setting = variable_get('update_check_disabled', FALSE);
  variable_set('update_check_disabled', TRUE);
  variable_set('update_last_check', 0);
  $result = drd_server_domain_list_updates('server');
  variable_set('update_check_disabled', $org_setting);

  // Just return result as is, it has been processed already
  return $result;
}

/**
 * DRD Server Operation to retrieve the SVN status (not yet implemented.
 *
 * @return type
 */
function drd_server_server_svn_status($option) {
  $def = _drd_server_svn_modules($option);
  $svn_enabled = variable_get('drd_server_svn_enabled', FALSE);
  $svn_user = variable_get('drd_server_svn_username', '');
  $svn_password = variable_get('drd_server_svn_password', '');
  $force_external = variable_get('drd_server_svn_force_external', TRUE);
  return drd_server_error('Not yet implemented');
  drupal_set_time_limit(0);
  $status = array();
  if (function_exists('svn_status') && ($force_external === FALSE)) {
    svn_auth_set_parameter(SVN_AUTH_PARAM_DEFAULT_USERNAME, $svn_user);
    svn_auth_set_parameter(SVN_AUTH_PARAM_DEFAULT_PASSWORD, $svn_password);
    $main = svn_status(DRUPAL_ROOT, SVN_NON_RECURSIVE + SVN_ALL);
    $mod_local = svn_status(DRUPAL_ROOT);
    $revision = $main[0]['revision'];
    foreach ($mod_local as $line) {
      if (is_dir($line['path']) && strpos('._', substr(basename($line['path']), 0, 1)) >= 0) {
        //ignore .DAV, .svn, _svn, etc. directory names
      }
      else {
        $status[] = array(
          'file_path' => $line['path'],
          'file_revision' => $line['revision'],
          'file_status' => $line['text_status'],
          'file_out_of_date' => (($line['text_status'] != SVN_WC_STATUS_NORMAL) || (($line['repos_text_status'] != SVN_WC_STATUS_NORMAL))),
        );
      }
    }
    $status[] = array('info' => 'SVN status collected by PHP function',);
  }
  else {
    // We should check if the PHP process has sufficient permissions to run
    // svn which most likely isn't the case. Then the resolution could be to
    // write a shell script which will then be executed by a cron tab job
    $cmd = "svn status -u ". DRUPAL_ROOT ." --username $svn_user --password $svn_password";
    $result = explode(':', exec($cmd, $output, $cmd_status));
    if ($cmd_status) {
      $cmd = "/usr/bin/". $cmd;
      $result = explode(':', exec($cmd, $output, $cmd_status));
    }
    $revision = (sizeof($result) > 1) ? trim(str_replace('.', '', $result[1])) : '';
    $revision = (is_numeric($revision)) ? $revision : t('Unknown');
    $next_line_conflict = FALSE;
    array_pop($output);
    foreach ($output as $line) {
      if (!$next_line_conflict) {
        $file_status = substr($line, 0, 7);
        $file_out_of_date = substr($line, 8, 1);
        $line = trim(substr($line, 9));
        $pos = strpos($line, ' ');
        $file_revision = trim(substr($line, 0, $pos));
        $file_path = trim(substr($line, $pos));
        $next_line_conflict = (substr($file_status, 6) == 'C');
        $status[] = array(
          'file_path' => $file_path,
          'file_revision' => $file_revision,
          'file_status' => $file_status,
          'file_out_of_date' => ($file_out_of_date == '*'),
        );
      }
      else {
        //ToDo: add conflict info to previous record
        $next_line_conflict = FALSE;
      }
    }
    $status[] = array('info' => 'SVN status collected by commandline tool',);
  }
  return drd_server_result('server.svn.status', array($revision, $status));
}

/**
 * DRD Server Operation to perform a SVN update (not yet implemented.
 *
 * @return type
 */
function drd_server_server_svn_update($svn_user, $svn_password, $force_external = FALSE) {
  return drd_server_error('Not yet implemented');
  drupal_set_time_limit(0);
  $status = array();
  if (function_exists('svn_update') && ($force_external === FALSE)) {
    svn_auth_set_parameter(SVN_AUTH_PARAM_DEFAULT_USERNAME, $svn_user);
    svn_auth_set_parameter(SVN_AUTH_PARAM_DEFAULT_PASSWORD, $svn_password);
    ob_start();
    $revision = svn_update(DRUPAL_ROOT);
    $output = ob_get_contents();
    ob_end_clean();
    $status[] = $output;
    $status[] = array('info' => 'SVN update performed by PHP function',);
  }
  else {
    $cmd = "svn update ". DRUPAL_ROOT ." --accept theirs-full --force --username $svn_user --password $svn_password --non-interactive";
    $result = explode(' ', exec($cmd, $output, $cmd_status));
    if ($cmd_status) {
      $cmd = "/usr/bin/". $cmd;
      $result = explode(':', exec($cmd, $output, $cmd_status));
    }
    $revision = trim(str_replace('.', '', array_pop($result)));
    array_pop($output);
    foreach ($output as $line) {
      $file_status = substr($line, 0, 3);
      $line = trim(substr($line, 5));
      $file_path = trim($line);
      $status[] = array(
        'file_path' => $file_path,
        'file_status' => $file_status,
      );
    }
    $status[] = array('info' => 'SVN update performed by commandline tool',);
  }
  return drd_server_result('server.svn.update', array($revision, $status));
}

/**
 * DRD Server Operation to retrieve the PHP error log from the OS.
 *
 * @return type
 */
function drd_server_server_php_error_log($max_length = 5000) {
  $log_file = ini_get('error_log');
  if (!file_exists($log_file)) {
    return drd_server_error(t('PHP Error Log does not exist.'), DRD_ERROR_NO_LOG_PHP);
  }
  $offset = max(-1, (filesize($log_file) - $max_length));
  $log = file_get_contents($log_file, FILE_BINARY, NULL, $offset);
  $result['cache']['php_error_log']['server'] = array(
    'title' => 'PHP Error Log',
    'callback' => 'drupal_render',
    'arguments' => array(
      'data' => array(
        '#markup' => $log,
        '#prefix' => '<pre>',
        '#suffix' => '</pre>',
      ),
    ),
  );
  return drd_server_result('server.php.error.log', $result);
}
